! Comment line to be replaced

!***********************************************************************

! Test program for the spline_derivative array valued function.   The
! spline derivative of a 3rd order function should be exact, within
! numerical round off.

    use splines                         ! the splines module

    implicit none                       ! be strict
    integer, parameter :: m=5           ! number of test points

    real, dimension(m) :: x, f, d, q
    real :: dx, p, eps
    integer :: i
    character :: stat*7, id*80 = '$Id: derivative_test.f90,v 1.5 2003/10/19 23:08:12 aake Exp $'

!-----------------------------------------------------------------------

    print *,id

    dx = 0.4
    x = dx*(/(float(i)/m, i=1,m)/)      ! a linear scale
    p = 3                               ! p'th order polynomium
    f = x**p                            ! function
    d = p*x**(p-1)                      ! derivative

! The maximum absolute value of the error

    eps = maxval(abs(spline_derivative(x,f,m)-d))

! Check if OK (2e-6 is an empirical value)

    if (eps < 2e-6) then
      stat = 'OK'
    else
      stat = 'NOT ok'
    end if

    print '(/1x,a,1pe9.2/1x,a/)', 'The maximum error is =', &
      eps, 'The spline_derivative array-valued function is '//stat

    end
