! Comment line to be replaced

MODULE splines

CONTAINS
!***********************************************************************
function spline_derivative (x, f, n) 
! 
!  Find the derivatives at the knots (x) of a cubic spline with
!  continuous second order derivatives, passsing through the function
!  values (f).  The boundary conditions on the spline are continuous
!  third derivatives at k=2 and k=2n-1.
!
!  The routine needs a work array w(n,3) that is allocated on the stack.
!
!***********************************************************************
!
  implicit none
  integer n,k,kk
  real, dimension(n):: x, f
  real, dimension(n):: spline_derivative, d
  real(kind=8), dimension(3,n):: w
  real(kind=8) c,cxa,cxb,cxc,dfa,dfb,dfc
  character:: id*80 = '$Id: splines.f90,v 1.8 2008/05/20 10:08:34 aake Exp $'

!
! first point
!
  if (n.eq.1) then
    d(1)=0.0
    spline_derivative=d
    return
  endif
  cxb=1./(x(2)-x(1))
  cxc=1./(x(3)-x(2))
  dfb=f(2)-f(1)
  dfc=f(3)-f(2)
  w(1,1)=cxb*cxb
  w(3,1)=-cxc*cxc
  w(2,1)=w(1,1)+w(3,1)
  d(1)=2.*(dfb*cxb*cxb*cxb-dfc*cxc*cxc*cxc)
!
! interior points
!
  do k=2,n-1
    cxa=1./(x(k)-x(k-1))
    cxb=1./(x(k+1)-x(k))
    dfa=f(k)-f(k-1)
    dfb=f(k+1)-f(k)
    w(1,k)=cxa
    w(3,k)=cxb
    w(2,k)=2.*(cxa+cxb)
    d(k)=3.*(dfb*cxb*cxb+dfa*cxa*cxa)
  end do
!
! last point
!
  w(1,n)=cxa*cxa
  w(3,n)=-cxb*cxb
  w(2,n)=w(1,n)+w(3,n)
  d(n)=2.*(dfa*cxa*cxa*cxa-dfb*cxb*cxb*cxb)
!
! eliminate at first point
!
  c=-w(3,1)/w(3,2)
  w(1,1)=w(1,1)+c*w(1,2)
  w(2,1)=w(2,1)+c*w(2,2)
  d(1)=d(1)+c*d(2)
  w(3,1)=w(2,1)
  w(2,1)=w(1,1)
!
! eliminate at last point
!
  c=-w(1,n)/w(1,n-1)
  w(2,n)=w(2,n)+c*w(2,n-1)
  w(3,n)=w(3,n)+c*w(3,n-1)
  d(n)=d(n)+c*d(n-1)
  w(1,n)=w(2,n)
  w(2,n)=w(3,n)
!
! eliminate subdiagonal
!
  do k=2,n
    c=-w(1,k)/w(2,k-1)
    w(2,k)=w(2,k)+c*w(3,k-1)
    d(k)=d(k)+c*d(k-1)
  end do
!
! backsubstitute
!
  d(n)=d(n)/w(2,n)
  do kk=2,n
    k=(n+1)-kk
    d(k)=(d(k)-w(3,k)*d(k+1))/w(2,k)
  end do
!
  spline_derivative=d
  return
end function spline_derivative

END MODULE splines
